local http = require('http')

local config = {}

function error(message, err)
	print("Sh_ONTIME_HANDLER", "ERROR", message, err)
end

function info(message)
	print("Sh_ONTIME_HANDLER", "INFO", message)
end

local function readFile(path)
	local open = io.open
    local file = open(path, "rb") -- r read mode and b binary mode
    if not file then return nil end
    local content = file:read "*a" -- *a or *all reads the whole file
    file:close()
    return content
end

function readConfig()
	config = require(scriptPath():gsub("sh_ontime_handler_avorion/main.lua", "config"))
	if config == nil or config.apiURL == nil or config.apiKey == nil then
		error("Invalid config!")
		exit()
	end
	info("Config loaded")
end

local function getUserJoinedMutation(id)
	local json = require('json')
    return json.encode({
        query = "mutation { userJoinedGameServer { userJoined(input: { ident = \"" .. id .. "\" }) { id } } }"
    })
end

local function getUserLeftMutation(id)
	local json = require('json')
    return json.encode({
        query = "mutation { userJoinedGameServer { userLeft(input: { ident = \"" .. id .. "\" }) { id } } }"
    })
end

function sendJoinedPlayer(id)
    local client = http.client()
    local request = http.request.new()
    request.uri = config.ApiURL
    request.method = "POST"
    request.headers["Authorization"] = "API-Key " .. config.ApiKey
    request.headers["Content-Type"] = "application/json"
    request.body = getUserJoinedMutation(id)
    local response = client:send(request)
    local success, result = pcall(json.parse, response.body)
    if success then
        local joinId = tonumber(result.data.userJoinedGameServer.userJoined.id)
    else
		error("Cannot send player joined", "")
    end
end

function sendLeftPlayer(id)
    local client = http.client()
    local request = http.request.new()
    request.uri = config.ApiURL
    request.method = "POST"
    request.headers["Authorization"] = "API-Key " .. config.ApiKey
    request.headers["Content-Type"] = "application/json"
    request.body = getUserLeftMutation(id)
    local response = client:send(request)
    local success, result = pcall(json.parse, response.body)
    if success then
        local joinId = tonumber(result.data.userJoinedGameServer.userJoined.id)
    else
		error("Cannot send player left", "")
    end
end

function initialize()
    Server():registerCallback("onPlayerLogIn", "handlePlayerJoined")
    Server():registerCallback("onPlayerLogOff", "handlePlayerLeft")
	info("Loading")
	readConfig()
	info("Listening")
end

function handlePlayerJoined(playerIndex)
	local player = Player(playerIndex)
    info(string.format("Player %s connected", player.name))
	sendJoinedPlayer(player.id)
end

function handlePlayerLeft(playerIndex)
	local player = Player(playerIndex)
    info(string.format("Player %s disconnected", player.name))
	-- sendJoinedPlayer(data.networkid)
end